@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -23,10 +23,9 @@ import contextlib
 import datetime
 import inspect
 import os
-import threading
 import warnings
 from enum import Enum, IntEnum, unique
-from typing import Any, Dict, List, Optional
+from typing import TYPE_CHECKING, Any, Collection, Dict, List, Optional, Union
 
 import attr
 
@@ -40,6 +39,9 @@ from hypothesis.internal.validation import check_type, try_convert
 from hypothesis.utils.conventions import not_set
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
+if TYPE_CHECKING:
+    from hypothesis.database import ExampleDatabase
+
 __all__ = ["settings"]
 
 all_settings = {}  # type: Dict[str, Setting]
@@ -65,7 +67,7 @@ class settingsProperty:
                     result = ExampleDatabase(not_set)
                 return result
             except KeyError:
-                raise AttributeError(self.name)
+                raise AttributeError(self.name) from None
 
     def __set__(self, obj, value):
         obj.__dict__[self.name] = value
@@ -129,7 +131,6 @@ class settings(metaclass=settingsMeta):
     changes made there will be picked up in newly created settings.
     """
 
-    _WHITELISTED_REAL_PROPERTIES = ["_construction_complete", "storage"]
     __definitions_are_locked = False
     _profiles = {}  # type: dict
     __module__ = "hypothesis"
@@ -140,35 +141,45 @@ class settings(metaclass=settingsMeta):
         else:
             raise AttributeError("settings has no attribute %s" % (name,))
 
-    def __init__(self, parent: Optional["settings"] = None, **kwargs: Any) -> None:
-        if parent is not None and not isinstance(parent, settings):
-            raise InvalidArgument(
-                "Invalid argument: parent=%r is not a settings instance" % (parent,)
-            )
-        if kwargs.get("derandomize"):
-            if kwargs.get("database") is not None:
+    def __init__(
+        self,
+        parent: Optional["settings"] = None,
+        *,
+        # This looks pretty strange, but there's good reason: we want Mypy to detect
+        # bad calls downstream, but not to freak out about the `= not_set` part even
+        # though it's not semantically valid to pass that as an argument value.
+        # The intended use is "like **kwargs, but more tractable for tooling".
+        max_examples: int = not_set,  # type: ignore
+        derandomize: bool = not_set,  # type: ignore
+        database: Union[None, "ExampleDatabase"] = not_set,  # type: ignore
+        verbosity: "Verbosity" = not_set,  # type: ignore
+        phases: Collection["Phase"] = not_set,  # type: ignore
+        stateful_step_count: int = not_set,  # type: ignore
+        report_multiple_bugs: bool = not_set,  # type: ignore
+        suppress_health_check: Collection["HealthCheck"] = not_set,  # type: ignore
+        deadline: Union[None, int, float, datetime.timedelta] = not_set,  # type: ignore
+        print_blob: bool = not_set,  # type: ignore
+    ) -> None:
+        if parent is not None:
+            check_type(settings, parent, "parent")
+        if derandomize not in (not_set, False):
+            if database not in (not_set, None):
                 raise InvalidArgument(
                     "derandomize=True implies database=None, so passing "
-                    "database=%r too is invalid." % (kwargs["database"],)
+                    f"database={database!r} too is invalid."
                 )
-            kwargs["database"] = None
-        self._construction_complete = False
+            database = None
+
         defaults = parent or settings.default
         if defaults is not None:
             for setting in all_settings.values():
-                if kwargs.get(setting.name, not_set) is not_set:
-                    kwargs[setting.name] = getattr(defaults, setting.name)
+                value = locals()[setting.name]
+                if value is not_set:
+                    object.__setattr__(
+                        self, setting.name, getattr(defaults, setting.name)
+                    )
                 else:
-                    if setting.validator:
-                        kwargs[setting.name] = setting.validator(kwargs[setting.name])
-        for name, value in kwargs.items():
-            if name not in all_settings:
-                raise InvalidArgument(
-                    "Invalid argument: %r is not a valid setting" % (name,)
-                )
-            setattr(self, name, value)
-        self.storage = threading.local()
-        self._construction_complete = True
+                    object.__setattr__(self, setting.name, setting.validator(value))
 
     def __call__(self, test):
         """Make the settings object (self) an attribute of the test.
@@ -242,6 +253,13 @@ class settings(metaclass=settingsMeta):
         if options is not None:
             options = tuple(options)
             assert default in options
+
+            def validator(value):
+                if value not in options:
+                    msg = f"Invalid {name}, {value!r}. Valid options: {options!r}"
+                    raise InvalidArgument(msg)
+                return value
+
         else:
             assert validator is not None
 
@@ -249,7 +267,6 @@ class settings(metaclass=settingsMeta):
             name=name,
             description=description.strip(),
             default=default,
-            options=options,
             validator=validator,
         )
         setattr(settings, name, settingsProperty(name, show_default))
@@ -259,24 +276,7 @@ class settings(metaclass=settingsMeta):
         settings.__definitions_are_locked = True
 
     def __setattr__(self, name, value):
-        if name in settings._WHITELISTED_REAL_PROPERTIES:
-            return object.__setattr__(self, name, value)
-        elif name in all_settings:
-            if self._construction_complete:
-                raise AttributeError(
-                    "settings objects are immutable and may not be assigned to"
-                    " after construction."
-                )
-            else:
-                setting = all_settings[name]
-                if setting.options is not None and value not in setting.options:
-                    raise InvalidArgument(
-                        "Invalid %s, %r. Valid options: %r"
-                        % (name, value, setting.options)
-                    )
-                return object.__setattr__(self, name, value)
-        else:
-            raise AttributeError("No such setting %s" % (name,))
+        raise AttributeError("settings objects are immutable")
 
     def __repr__(self):
         bits = ("%s=%r" % (name, getattr(self, name)) for name in all_settings)
@@ -318,7 +318,7 @@ class settings(metaclass=settingsMeta):
         try:
             return settings._profiles[name]
         except KeyError:
-            raise InvalidArgument("Profile %r is not registered" % (name,))
+            raise InvalidArgument(f"Profile {name!r} is not registered") from None
 
     @staticmethod
     def load_profile(name: str) -> None:
@@ -345,7 +345,6 @@ class Setting:
     name = attr.ib()
     description = attr.ib()
     default = attr.ib()
-    options = attr.ib()
     validator = attr.ib()
 
 
@@ -375,6 +374,9 @@ If you are writing one-off tests, running tens of thousands of examples is
 quite reasonable as Hypothesis may miss uncommon bugs with default settings.
 For very complex code, we have observed Hypothesis finding novel bugs after
 *several million* examples while testing :pypi:`SymPy`.
+If you are running more than 100k examples for a test, consider using our
+:ref:`integration for coverage-guided fuzzing <fuzz_one_input>` - it really
+shines when given minutes or hours to run.
 """,
 )
 
@@ -415,10 +417,12 @@ settings._define_setting(
     default=not_set,
     show_default=False,
     description="""
-An instance of hypothesis.database.ExampleDatabase that will be
+An instance of :class:`~hypothesis.database.ExampleDatabase` that will be
 used to save examples to and load previous examples from. May be ``None``
-in which case no storage will be used, ``":memory:"`` for an in-memory
-database, or any path for a directory-based example database.
+in which case no storage will be used.
+
+See the :doc:`example database documentation <database>` for a list of built-in
+example database implementations, and how to define custom implementations.
 """,
     validator=_validate_database,
 )
@@ -609,7 +613,7 @@ allowed to exceed. Tests which take longer than that may be converted into
 errors (but will not necessarily be if close to the deadline, to allow some
 variability in test run time).
 
-Set this to None to disable this behaviour entirely.
+Set this to ``None`` to disable this behaviour entirely.
 """,
 )
 
@@ -635,13 +639,28 @@ The default is ``True`` if the ``CI`` or ``TF_BUILD`` env vars are set, ``False`
 settings.lock_further_definitions()
 
 
-def note_deprecation(message: str, *, since: str) -> None:
+def note_deprecation(message: str, *, since: str, has_codemod: bool) -> None:
     if since != "RELEASEDAY":
         date = datetime.datetime.strptime(since, "%Y-%m-%d").date()
         assert datetime.date(2016, 1, 1) <= date
+    if has_codemod:
+        message += (
+            "\n    The `hypothesis codemod` command-line tool can automatically "
+            "refactor your code to fix this warning."
+        )
     warnings.warn(HypothesisDeprecationWarning(message), stacklevel=2)
 
 
 settings.register_profile("default", settings())
 settings.load_profile("default")
 assert settings.default is not None
+
+
+# Check that the kwonly args to settings.__init__ is the same as the set of
+# defined settings - in case we've added or remove something from one but
+# not the other.
+assert set(all_settings) == {
+    p.name
+    for p in inspect.signature(settings.__init__).parameters.values()
+    if p.kind == inspect.Parameter.KEYWORD_ONLY
+}
