@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -156,7 +156,7 @@ def seed(seed: Hashable) -> Callable[[TestFunc], TestFunc]:
     return accept
 
 
-def reproduce_failure(version, blob):
+def reproduce_failure(version: str, blob: bytes) -> Callable[[TestFunc], TestFunc]:
     """Run the example that corresponds to this data blob in order to reproduce
     a failure.
 
@@ -354,19 +354,38 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs):
                 # each of the (report text, error) pairs we find back to the top-level
                 # runner.  This also ensures that user-facing stack traces have as few
                 # frames of Hypothesis internals as possible.
-                yield (fragments_reported, err.with_traceback(get_trimmed_traceback()))
+                err = err.with_traceback(get_trimmed_traceback())
+
+                # One user error - whether misunderstanding or typo - we've seen a few
+                # times is to pass strategies to @example() where values are expected.
+                # Checking is easy, and false-positives not much of a problem, so:
+                if any(
+                    isinstance(arg, SearchStrategy)
+                    for arg in example.args + tuple(example.kwargs.values())
+                ):
+                    new = HypothesisWarning(
+                        "The @example() decorator expects to be passed values, but "
+                        "you passed strategies instead.  See https://hypothesis."
+                        "readthedocs.io/en/latest/reproducing.html for details."
+                    )
+                    new.__cause__ = err
+                    err = new
+
+                yield (fragments_reported, err)
                 if state.settings.report_multiple_bugs:
                     continue
                 break
             finally:
-                assert fragments_reported[0].startswith("Falsifying example")
-                fragments_reported[0] = fragments_reported[0].replace(
-                    "Falsifying example", "Falsifying explicit example", 1
-                )
+                if fragments_reported:
+                    assert fragments_reported[0].startswith("Falsifying example")
+                    fragments_reported[0] = fragments_reported[0].replace(
+                        "Falsifying example", "Falsifying explicit example", 1
+                    )
 
-            verbose_report(fragments_reported[0].replace("Falsifying", "Trying", 1))
-            for f in fragments_reported[1:]:
-                verbose_report(f)
+            if fragments_reported:
+                verbose_report(fragments_reported[0].replace("Falsifying", "Trying", 1))
+                for f in fragments_reported[1:]:
+                    verbose_report(f)
 
 
 def get_random_for_wrapped_test(test, wrapped_test):
@@ -1170,6 +1189,11 @@ def given(
                 test_runner, search_strategy, test, settings, random, wrapped_test
             )
             digest = function_digest(test)
+            # We track the minimal-so-far example for each distinct origin, so
+            # that we track log-n instead of n examples for long runs.  In particular
+            # it means that we saturate for common errors in long runs instead of
+            # storing huge volumes of low-value data.
+            minimal_failures: dict = {}
 
             def fuzz_one_input(
                 buffer: Union[bytes, bytearray, memoryview, BinaryIO]
@@ -1185,8 +1209,13 @@ def given(
                 except (StopTest, UnsatisfiedAssumption):
                     return None
                 except BaseException:
-                    if settings.database is not None:
-                        settings.database.save(digest, bytes(data.buffer))
+                    buffer = bytes(data.buffer)
+                    known = minimal_failures.get(data.interesting_origin)
+                    if settings.database is not None and (
+                        known is None or sort_key(buffer) <= sort_key(known)
+                    ):
+                        settings.database.save(digest, buffer)
+                        minimal_failures[data.interesting_origin] = buffer
                     raise
                 return bytes(data.buffer)
 
