@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -34,7 +34,7 @@ from hypothesis.strategies._internal import core as st
 from hypothesis.strategies._internal.ipaddress import ip_addresses
 from hypothesis.strategies._internal.strategies import SearchStrategy
 
-URL_SAFE_CHARACTERS = frozenset(string.ascii_letters + string.digits + "$-_.+!*'(),")
+URL_SAFE_CHARACTERS = frozenset(string.ascii_letters + string.digits + "$-_.+!*'(),~")
 
 
 # This file is sourced from http://data.iana.org/TLD/tlds-alpha-by-domain.txt
@@ -167,6 +167,7 @@ def ip4_addr_strings() -> SearchStrategy[str]:
         "Use `ip_addresses(v=4).map(str)` instead of `ip4_addr_strings()`; "
         "the provisional strategy is less flexible and will be removed.",
         since="2020-01-21",
+        has_codemod=False,
     )
     return ip_addresses(v=4).map(str)
 
@@ -177,5 +178,6 @@ def ip6_addr_strings() -> SearchStrategy[str]:
         "Use `ip_addresses(v=6).map(str)` instead of `ip6_addr_strings()`; "
         "the provisional strategy is less flexible and will be removed.",
         since="2020-01-21",
+        has_codemod=False,
     )
     return ip_addresses(v=6).map(str)
